package ${domain};

import ${domain}.frame.auth.Token;
import ${domain}.frame.base.BaseResponse;
import ${domain}.frame.utils.MapperUtil;
import ${domain}.module.wsys.ent.Res;
import ${domain}.module.wsys.mgr.DictItemManager;
import ${domain}.module.wsys.mgr.DictManager;
import ${domain}.module.wsys.mgr.ResManager;
import ${domain}.module.wsys.mgr.RoleManager;
import ${domain}.module.wsys.mgr.RoleResManager;
import ${domain}.module.wsys.req.DictCreateRequest;
import ${domain}.module.wsys.req.DictFindRequest;
import ${domain}.module.wsys.req.DictItemCreateRequest;
import ${domain}.module.wsys.req.DictItemFindRequest;
import ${domain}.module.wsys.req.ResCreateRequest;
import ${domain}.module.wsys.req.ResFindRequest;
import ${domain}.module.wsys.req.RoleCreateRequest;
import ${domain}.module.wsys.req.RoleFindRequest;
import ${domain}.module.wsys.req.RoleResCheckRequest;
import ${domain}.module.wsys.rsp.DictCreateResponse;
import ${domain}.module.wsys.rsp.DictFindResponse;
import ${domain}.module.wsys.rsp.DictItemCreateResponse;
import ${domain}.module.wsys.rsp.DictItemFindResponse;
import ${domain}.module.wsys.rsp.ResCreateResponse;
import ${domain}.module.wsys.rsp.ResFindResponse;
import ${domain}.module.wsys.rsp.RoleCreateResponse;
import ${domain}.module.wsys.rsp.RoleFindResponse;
import ${domain}.module.wsys.rsp.RoleResCheckResponse;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static org.junit.Assert.assertTrue;

@RunWith(SpringRunner.class)
@SpringBootTest
@Transactional
public class DataInit {

    @Autowired
    private Token token;
    @Autowired
    private DictManager dictManager;
    @Autowired
    private DictItemManager dictItemManager;
    @Autowired
    private ResManager resManager;
    @Autowired
    private RoleManager roleManager;
    @Autowired
    private RoleResManager roleCreateRequest;

    /**
     * 字典数据初始化
     */
    @Test
    @Rollback(false)
    public void init() {
        //region 字典初始化
        {
<#list project.modules as module>
    <#list module.tables as table>
        <#list table.fields as field>
            <#if field.fieldType == 'Dict' && module.moduleName != 'wsys'>
            {// ${field.fieldComment?default('')}
                createDict("${field.fieldName?default('')}", "${field.fieldComment?default('')}", new String[][]{
                    <#list field.selectItems as item>
                        {"${item.value?default('')}", "${item.label?default('')}"},
                    </#list>
                });
            }
            </#if>
        </#list>
    </#list>
</#list>
        }
        //endregion

        //region 管理员初始化
        {
            List<Res> forDev = new ArrayList<>();
            {// 初始化开发角色特有资源
                forDev.add(createRes("01", "核心设置菜单", "WEB", "DEV"));
                forDev.add(createRes("0101", "系统配置菜单", "WEB", "DEV_PROFILES"));
                forDev.add(createRes("010101", "系统配置管理页", "WEB", "/wsys/profiles.htm"));
                forDev.add(createRes("01010101", "系统配置模板", "WEB", "/ajax/wsys/profiles/template"));
                forDev.add(createRes("01010102", "系统配置导入", "WEB", "/ajax/wsys/profiles/imports"));
                forDev.add(createRes("01010103", "系统配置导出", "WEB", "/ajax/wsys/profiles/exports"));
                forDev.add(createRes("01010104", "系统配置新增", "WEB", "/ajax/wsys/profiles/create"));
                forDev.add(createRes("01010105", "系统配置删除", "WEB", "/ajax/wsys/profiles/delete"));
                forDev.add(createRes("01010106", "系统配置编辑", "WEB", "/ajax/wsys/profiles/update"));
                forDev.add(createRes("01010107", "系统配置查询", "WEB", "/ajax/wsys/profiles/find"));
                forDev.add(createRes("01010108", "系统配置获取", "WEB", "/ajax/wsys/profiles/get"));
                forDev.add(createRes("0102", "资源管理菜单", "WEB", "DEV_RES"));
                forDev.add(createRes("010201", "资源管理页", "WEB", "/wsys/res.htm"));
                forDev.add(createRes("01020101", "资源模板", "WEB", "/ajax/wsys/res/template"));
                forDev.add(createRes("01020102", "资源导入", "WEB", "/ajax/wsys/res/imports"));
                forDev.add(createRes("01020103", "资源导出", "WEB", "/ajax/wsys/res/exports"));
                forDev.add(createRes("01020104", "资源新增", "WEB", "/ajax/wsys/res/create"));
                forDev.add(createRes("01020105", "资源删除", "WEB", "/ajax/wsys/res/delete"));
                forDev.add(createRes("01020106", "资源编辑", "WEB", "/ajax/wsys/res/update"));
                forDev.add(createRes("01020107", "资源查询", "WEB", "/ajax/wsys/res/find"));
                forDev.add(createRes("01020108", "资源获取", "WEB", "/ajax/wsys/res/get"));
                forDev.add(createRes("0103", "SQL任务菜单", "WEB", "DEV_TASK_SQL"));
                forDev.add(createRes("010301", "SQL任务管理页", "WEB", "/wsys/taskSql.htm"));
                forDev.add(createRes("01030101", "SQL任务模板", "WEB", "/ajax/wsys/taskSql/template"));
                forDev.add(createRes("01030102", "SQL任务导入", "WEB", "/ajax/wsys/taskSql/imports"));
                forDev.add(createRes("01030103", "SQL任务导出", "WEB", "/ajax/wsys/taskSql/exports"));
                forDev.add(createRes("01030104", "SQL任务新增", "WEB", "/ajax/wsys/taskSql/create"));
                forDev.add(createRes("01030105", "SQL任务删除", "WEB", "/ajax/wsys/taskSql/delete"));
                forDev.add(createRes("01030106", "SQL任务编辑", "WEB", "/ajax/wsys/taskSql/update"));
                forDev.add(createRes("01030107", "SQL任务查询", "WEB", "/ajax/wsys/taskSql/find"));
                forDev.add(createRes("01030108", "SQL任务获取", "WEB", "/ajax/wsys/taskSql/get"));
                forDev.add(createRes("0104", "应用接入管理菜单", "WEB", "DEV_VISITOR"));
                forDev.add(createRes("010401", "应用接入管理页", "WEB", "/wsys/visitor.htm"));
                forDev.add(createRes("01040101", "应用接入模板", "WEB", "/ajax/wsys/visitor/template"));
                forDev.add(createRes("01040102", "应用接入导入", "WEB", "/ajax/wsys/visitor/imports"));
                forDev.add(createRes("01040103", "应用接入导出", "WEB", "/ajax/wsys/visitor/exports"));
                forDev.add(createRes("01040104", "应用接入新增", "WEB", "/ajax/wsys/visitor/create"));
                forDev.add(createRes("01040105", "应用接入删除", "WEB", "/ajax/wsys/visitor/delete"));
                forDev.add(createRes("01040106", "应用接入更新", "WEB", "/ajax/wsys/visitor/update"));
                forDev.add(createRes("01040107", "应用接入查询", "WEB", "/ajax/wsys/visitor/find"));
                forDev.add(createRes("01040108", "应用资源关系新增", "WEB", "/ajax/wsys/visitorRes/create"));
                forDev.add(createRes("01040109", "应用资源关系删除", "WEB", "/ajax/wsys/visitorRes/delete"));
                forDev.add(createRes("01040110", "应用资源关系更新", "WEB", "/ajax/wsys/visitorRes/update"));
                forDev.add(createRes("01040111", "应用资源关系查询", "WEB", "/ajax/wsys/visitorRes/find"));

                forDev.add(createRes("02", "监控运行菜单", "WEB", "MNT"));
                forDev.add(createRes("0201", "登录管理菜单", "WEB", "MNT_TOKENS"));
                forDev.add(createRes("020101", "登录管理页", "WEB", "/wsys/tokens.htm"));
                forDev.add(createRes("02010101", "登录查询", "WEB", "/ajax/wsys/tokens/find"));
                forDev.add(createRes("02010102", "登录导出", "WEB", "/ajax/wsys/tokens/exports"));
                forDev.add(createRes("02010103", "登录注销", "WEB", "/ajax/wsys/tokens/logout"));
                forDev.add(createRes("0202", "调度监控菜单", "WEB", "MNT_SCHEDULE"));
                forDev.add(createRes("020201", "调度监控页", "WEB", "/wsys/schedule.htm"));
                forDev.add(createRes("02020101", "调度监控查询", "WEB", "/ajax/wsys/schedule/find"));
                forDev.add(createRes("02020102", "调度监控启动", "WEB", "/ajax/wsys/schedule/start"));
                forDev.add(createRes("02020103", "调度监控暂停", "WEB", "/ajax/wsys/schedule/stop"));
                forDev.add(createRes("0203", "错误日志菜单", "WEB", "MNT_LOGERR"));
                forDev.add(createRes("020301", "错误日志管理页", "WEB", "/wsys/logerr.htm"));
                forDev.add(createRes("02030101", "错误日志删除", "WEB", "/ajax/wsys/logerr/delete"));
                forDev.add(createRes("02030102", "错误日志更新", "WEB", "/ajax/wsys/logerr/update"));
                forDev.add(createRes("02030103", "错误日志查询", "WEB", "/ajax/wsys/logerr/find"));
            }

            List<Res> forMgr = new ArrayList<>();
            {// 初始化管理资源
                forMgr.add(createRes("03", "基础设置菜单", "WEB", "SYS"));

                forMgr.add(createRes("0301", "根页", "WEB", "/", true));
                forMgr.add(createRes("030101", "主页", "WEB", "/index.htm", true));
                forMgr.add(createRes("030102", "首页", "WEB", "/home.htm", true));

                forMgr.add(createRes("0302", "字典管理菜单", "WEB", "SYS_DICT"));
                forMgr.add(createRes("030201", "字典管理页", "WEB", "/wsys/dict.htm"));
                forMgr.add(createRes("03020101", "字典模板", "WEB", "/ajax/wsys/dict/template"));
                forMgr.add(createRes("03020102", "字典导入", "WEB", "/ajax/wsys/dict/imports"));
                forMgr.add(createRes("03020103", "字典导出", "WEB", "/ajax/wsys/dict/exports"));
                forMgr.add(createRes("03020104", "字典新建", "WEB", "/ajax/wsys/dict/create"));
                forMgr.add(createRes("03020105", "字典删除", "WEB", "/ajax/wsys/dict/delete"));
                forMgr.add(createRes("03020106", "字典更新", "WEB", "/ajax/wsys/dict/update"));
                forMgr.add(createRes("03020107", "字典查询", "WEB", "/ajax/wsys/dict/find"));
                forMgr.add(createRes("03020108", "字典获取", "WEB", "/ajax/wsys/dict/get"));
                forMgr.add(createRes("03020109", "字典详情", "WEB", "/ajax/wsys/dict/load", true));
                forMgr.add(createRes("03020110", "字典项管理页", "WEB", "/wsys/dictItem.htm"));
                forMgr.add(createRes("0302011001", "字典项模板", "WEB", "/ajax/wsys/dictItem/template"));
                forMgr.add(createRes("0302011002", "字典项导入", "WEB", "/ajax/wsys/dictItem/imports"));
                forMgr.add(createRes("0302011003", "字典项导出", "WEB", "/ajax/wsys/dictItem/exports"));
                forMgr.add(createRes("0302011004", "字典项新增", "WEB", "/ajax/wsys/dictItem/create"));
                forMgr.add(createRes("0302011005", "字典项删除", "WEB", "/ajax/wsys/dictItem/delete"));
                forMgr.add(createRes("0302011006", "字典项更新", "WEB", "/ajax/wsys/dictItem/update"));
                forMgr.add(createRes("0302011007", "字典项查找", "WEB", "/ajax/wsys/dictItem/find"));
                forMgr.add(createRes("0302011008", "字典项获取", "WEB", "/ajax/wsys/dictItem/get"));

                forMgr.add(createRes("0303", "用户管理菜单", "WEB", "SYS_USER"));
                forMgr.add(createRes("030301", "用户管理页", "WEB", "/wsys/user.htm"));
                forMgr.add(createRes("03030101", "字用户模板", "WEB", "/ajax/wsys/user/template"));
                forMgr.add(createRes("03030102", "用户导入", "WEB", "/ajax/wsys/user/imports"));
                forMgr.add(createRes("03030103", "用户导出", "WEB", "/ajax/wsys/user/exports"));
                forMgr.add(createRes("03030104", "用户新增", "WEB", "/ajax/wsys/user/create"));
                forMgr.add(createRes("03030105", "用户删除", "WEB", "/ajax/wsys/user/delete"));
                forMgr.add(createRes("03030106", "用户更新", "WEB", "/ajax/wsys/user/update"));
                forMgr.add(createRes("03030107", "用户查询", "WEB", "/ajax/wsys/user/find"));
                forMgr.add(createRes("03030108", "用户获取", "WEB", "/ajax/wsys/user/get"));

                forMgr.add(createRes("0304", "机构管理菜单", "WEB", "SYS_DEPT"));
                forMgr.add(createRes("030401", "机构管理页", "WEB", "/wsys/dept.htm"));
                forMgr.add(createRes("03040101", "机构新增", "WEB", "/ajax/wsys/dept/create"));
                forMgr.add(createRes("03040102", "机构删除", "WEB", "/ajax/wsys/dept/delete"));
                forMgr.add(createRes("03040103", "机构更新", "WEB", "/ajax/wsys/dept/update"));
                forMgr.add(createRes("03040104", "机构查询", "WEB", "/ajax/wsys/dept/find"));
                forMgr.add(createRes("03040105", "机构获取", "WEB", "/ajax/wsys/dept/get"));
                forMgr.add(createRes("03040106", "机构树查询", "WEB", "/ajax/wsys/dept/tree"));
                forMgr.add(createRes("03040107", "机构模板", "WEB", "/ajax/wsys/dept/template"));
                forMgr.add(createRes("03040108", "机构导入", "WEB", "/ajax/wsys/dept/imports"));
                forMgr.add(createRes("03040109", "机构导出", "WEB", "/ajax/wsys/dept/exports"));

                forMgr.add(createRes("0304", "文件管理菜单", "WEB", "SYS_FILE"));
                forMgr.add(createRes("030401", "文件管理页", "WEB", "/wsys/file.htm"));
                forMgr.add(createRes("03040101", "文件删除", "WEB", "/ajax/wsys/file/delete"));
                forMgr.add(createRes("03040102", "文件查询", "WEB", "/ajax/wsys/file/find"));

                forMgr.add(createRes("0305", "角色管理菜单", "WEB", "SYS_ROLE"));
                forMgr.add(createRes("030501", "角色管理页", "WEB", "/wsys/role.htm"));
                forMgr.add(createRes("03050101", "角色模板", "WEB", "/ajax/wsys/role/template"));
                forMgr.add(createRes("03050102", "角色导入", "WEB", "/ajax/wsys/role/imports"));
                forMgr.add(createRes("03050103", "角色导出", "WEB", "/ajax/wsys/role/exports"));
                forMgr.add(createRes("03050104", "角色新增", "WEB", "/ajax/wsys/role/create"));
                forMgr.add(createRes("03050105", "角色删除", "WEB", "/ajax/wsys/role/delete"));
                forMgr.add(createRes("03050106", "角色更新", "WEB", "/ajax/wsys/role/update"));
                forMgr.add(createRes("03050107", "角色查询", "WEB", "/ajax/wsys/role/find"));
                forMgr.add(createRes("03050108", "角色获取", "WEB", "/ajax/wsys/role/get"));
                forMgr.add(createRes("03050109", "角色资源分配", "WEB", "/ajax/wsys/roleRes/check"));
                forMgr.add(createRes("03050100", "角色资源关系查询", "WEB", "/ajax/wsys/roleRes/find"));
                forMgr.add(createRes("03050110", "资源树查询", "WEB", "/ajax/wsys/res/tree"));

<#list project.modules as module>
<#assign index = module_index + 4/>
<#if index<10>
<#assign index = "0"+index/>
</#if>
<#if module.getNeedGenerate()>
                forMgr.add(createRes("${index}", "${module.moduleComment}菜单", "WEB", "${module.moduleName?upper_case}"));
    <#list module.tables as table>
        <#if table.html>
                forMgr.add(createRes("${index}01", "${table.tableComment}管理菜单", "WEB", "${module.moduleName?upper_case}_${table.tableName?upper_case}"));
                forMgr.add(createRes("${index}0101", "${table.tableComment}管理页", "WEB", "/${module.moduleName}/${table.getFName()}.htm"));
                forMgr.add(createRes("${index}010101", "${table.tableComment}模板", "WEB", "/ajax/${module.moduleName}/${table.getFName()}/template"));
                forMgr.add(createRes("${index}010102", "${table.tableComment}导入", "WEB", "/ajax/${module.moduleName}/${table.getFName()}/imports"));
                forMgr.add(createRes("${index}010103", "${table.tableComment}导出", "WEB", "/ajax/${module.moduleName}/${table.getFName()}/exports"));
        </#if>
        <#if table.create>
                forMgr.add(createRes("${index}010104", "${table.tableComment}新增", "WEB", "/ajax/${module.moduleName}/${table.getFName()}/create"));
        </#if>
        <#if table.delete>
                forMgr.add(createRes("${index}010105", "${table.tableComment}删除", "WEB", "/ajax/${module.moduleName}/${table.getFName()}/delete"));
        </#if>
        <#if table.update>
                forMgr.add(createRes("${index}010106", "${table.tableComment}更新", "WEB", "/ajax/${module.moduleName}/${table.getFName()}/update"));
        </#if>
        <#if table.find>
                forMgr.add(createRes("${index}010107", "${table.tableComment}查询", "WEB", "/ajax/${module.moduleName}/${table.getFName()}/find"));
        </#if>
        <#if table.get>
                forMgr.add(createRes("${index}010108", "${table.tableComment}获取", "WEB", "/ajax/${module.moduleName}/${table.getFName()}/get"));
        </#if>
        <#if table.search>
                forMgr.add(createRes("${index}010109", "${table.tableComment}搜索", "WEB", "/ajax/${module.moduleName}/${table.getFName()}/search"));
        </#if>
    </#list>
</#if>
</#list>
            }
<#if project.isNeedApi()>

            {// API资源
                forMgr.add(createRes("A", "接口资源", "API", "A"));
                forMgr.add(createRes("A01", "基础资源", "API", "A01"));
                forMgr.add(createRes("A0101", "用户登录", "API", "/api/wsys/User/login"));
                forMgr.add(createRes("A0102", "字典查询", "API", "/api/wsys/Dict/find"));
                forMgr.add(createRes("A0103", "加载字典", "API", "/api/wsys/Dict/load"));
                forMgr.add(createRes("A0104", "文件上传", "API", "/api/wsys/File/upload"));
                forMgr.add(createRes("A0105", "文件下载", "API", "/api/wsys/File/download"));
<#list project.modules as module>
<#assign index = module_index + 2/>
<#if index<10>
<#assign index = "0"+index/>
</#if>
                forMgr.add(createRes("A${index}", "${module.moduleComment}", "API", "A${module.moduleName?upper_case}"));
    <#list module.tables as table>
                forMgr.add(createRes("A${index}01", "${table.tableComment}", "API", "A${module.moduleName?upper_case}_${table.tableName?upper_case}"));
        <#if table.create>
                forMgr.add(createRes("A${index}0101", "${table.tableComment}新增", "API", "/api/${module.moduleName}/${table.getFName()}/create"));
        </#if>
        <#if table.delete>
                forMgr.add(createRes("A${index}0102", "${table.tableComment}删除", "API", "/api/${module.moduleName}/${table.getFName()}/delete"));
        </#if>
        <#if table.update>
                forMgr.add(createRes("A${index}0103", "${table.tableComment}更新", "API", "/api/${module.moduleName}/${table.getFName()}/update"));
        </#if>
        <#if table.find>
                forMgr.add(createRes("A${index}0104", "${table.tableComment}查询", "API", "/api/${module.moduleName}/${table.getFName()}/find"));
        </#if>
        <#if table.get>
                forMgr.add(createRes("A${index}0105", "${table.tableComment}获取", "API", "/api/${module.moduleName}/${table.getFName()}/get"));
        </#if>
        <#if table.search>
                forMgr.add(createRes("A${index}0107", "${table.tableComment}搜索", "API", "/api/${module.moduleName}/${table.getFName()}/search"));
        </#if>
    </#list>
</#list>
            }
</#if>

            {//初始化开发角色
                Long roleId = createRole("developer", "开发", "开发");
                for (Res res : forDev) {
                    allocateRes(roleId, "developer", res.getId(), res.getResCode());
                }
                for (Res res : forMgr) {
                    allocateRes(roleId, "developer", res.getId(), res.getResCode());
                }
            }
            {//初始化管理角色
                Long roleId = createRole("management", "管理员", "管理员");
                for (Res res : forMgr) {
                    allocateRes(roleId, "management", res.getId(), res.getResCode());
                }
            }
        }
        //endregion
    }

    private void createDict(String name, String comment, String[][] items) {
        DictFindRequest dictFindRequest = new DictFindRequest();
        dictFindRequest.setDictName(name);
        DictFindResponse dictFindResponse = dictManager.find(dictFindRequest, token);
        if (dictFindResponse.getResult().size() == 0) {
            DictCreateRequest dictCreateRequest = new DictCreateRequest();
            dictCreateRequest.setDictName(name);
            dictCreateRequest.setDictComment(comment);
            dictCreateRequest.setVersion(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));
            dictCreateRequest.setValid(true);
            DictCreateResponse dictCreateResponse = dictManager.create(dictCreateRequest, token);
            printError(dictCreateResponse);
            assertTrue(!dictCreateResponse.hasError());
        }

        // 通过查询获取字典项已有数量
        DictItemFindRequest dictItemFindRequest = new DictItemFindRequest();
        dictItemFindRequest.setDictName(name);
        DictItemFindResponse dictItemFindResponse = dictItemManager.find(dictItemFindRequest, token);

        for (int i = 0; i < items.length; i++) {
            String[] item = items[i];
            dictItemFindRequest.setKey(item[0]);
            DictItemFindResponse dictItemFindResponse_ = dictItemManager.find(dictItemFindRequest, token);
            if (dictItemFindResponse_.getResult().size() == 0) {
                DictItemCreateRequest dictItemCreateRequest = new DictItemCreateRequest();
                dictItemCreateRequest.setDictName(name);
                dictItemCreateRequest.setKey(item[0]);
                dictItemCreateRequest.setValue(item[1]);
                dictItemCreateRequest.setSort((int) (i + dictItemFindResponse.getTotalCount()));
                dictItemCreateRequest.setValid(true);
                DictItemCreateResponse dictItemCreateResponse = dictItemManager.create(dictItemCreateRequest, token);
                printError(dictItemCreateResponse);
                assertTrue(!dictItemCreateResponse.hasError());
            }
        }
    }

    private Res createRes(String code, String name, String resType, String value) {
        return createRes(code, name, resType, value, false);
    }

    private Res createRes(String code, String name, String resType, String value, boolean free) {
        ResFindRequest resFindRequest = new ResFindRequest();
        resFindRequest.setResCode(code);
        ResFindResponse resFindResponse = resManager.find(resFindRequest, token);
        if (resFindResponse.getResult().size() == 0) {
            ResCreateRequest resCreateRequest = new ResCreateRequest();
            resCreateRequest.setResName(name);
            resCreateRequest.setResCode(code);
            resCreateRequest.setResType(resType);
            resCreateRequest.setResValue(value);
            resCreateRequest.setFree(free);
            resCreateRequest.setValid(true);
            ResCreateResponse resCreateResponse = resManager.create(resCreateRequest, token);
            printError(resCreateResponse);
            assertTrue(!resCreateResponse.hasError());

            Res res = new Res();
            res.setResCode(code);
            res.setResName(name);
            res.setResType(resType);
            res.setId(resCreateResponse.getId());
            return res;
        } else {
            return resFindResponse.getResult().get(0);
        }
    }

    private Long createRole(String roleCode, String roleName, String roleComment) {
        RoleFindRequest roleFindRequest = new RoleFindRequest();
        roleFindRequest.setCode(roleCode);
        RoleFindResponse roleFindResponse = roleManager.find(roleFindRequest, token);
        if (roleFindResponse.getResult().size() == 0) {
            RoleCreateRequest roleCreateRequest = new RoleCreateRequest();
            roleCreateRequest.setCode(roleCode);
            roleCreateRequest.setName(roleName);
            roleCreateRequest.setComment(roleComment);
            RoleCreateResponse roleCreateResponse = roleManager.create(roleCreateRequest, token);
            printError(roleCreateResponse);
            assertTrue(!roleCreateResponse.hasError());
            return roleCreateResponse.getId();
        } else {
            return roleFindResponse.getResult().get(0).getId();
        }
    }

    private void allocateRes(Long roleId, String roleCode, Long resId, String resCode) {
        RoleResCheckRequest roleResCheckRequest = new RoleResCheckRequest();
        roleResCheckRequest.setRoleId(roleId);
        roleResCheckRequest.setRoleCode(roleCode);
        roleResCheckRequest.setResId(resId);
        roleResCheckRequest.setResCode(resCode);
        roleResCheckRequest.setChecked(true);

        RoleResCheckResponse roleResCheckResponse = roleCreateRequest.check(roleResCheckRequest, token);
        printError(roleResCheckResponse);
        assertTrue(!roleResCheckResponse.hasError());
    }

    private void printError(BaseResponse response) {
        if (response.hasError()) {
            System.err.println("==========>" + MapperUtil.toJson(response));
        }
    }
}